/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from "react";
import moment from 'moment';
// Maerial ui components
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import Paper from "@material-ui/core/Paper";
import Grid from "@material-ui/core/Grid";
import RemoveCircleOutlineRoundedIcon from '@material-ui/icons/RemoveCircleOutlineRounded';
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import Chip from '@material-ui/core/Chip';
import { useStyles, StyledCard } from "../../Styles/DataCard";
import EmailRoundedIcon from '@material-ui/icons/EmailRounded';
import EditRoundedIcon from '@material-ui/icons/EditRounded';
import DeleteRoundedIcon from '@material-ui/icons/DeleteRounded';
import CheckRoundedIcon from "@material-ui/icons/CheckRounded";
import CloseRoundedIcon from "@material-ui/icons/CloseRounded";
//imports from local files
import { GetAData } from "../../../Filters";
import { apiDomain } from '../../../../env';
import { Link, useHistory } from "react-router-dom";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import SnackBar from '../../../../Components/SnackbarComponent';
import { getRequest, postRequest, deleteRequest, putRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { Store } from 'Store'
import StyledButton from '../Button';
//styles
import styled from 'styled-components';
//Image Import
import { noAvatar } from "ImportImages.js";

const CollectionCard = () => {

    const { state } = useContext(Store);

    let history = useHistory()
    const classes = useStyles();
    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("id");

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({});

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const stateTeacherRequest = (status) => {
        switch (status) {
            case "accepted":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: state.contrast === "" ? "#228B22" : "black",
                            border: state.contrast === "" ? "" : "1px solid white",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        ACEITO
                    </Paper>
                );
            case "requested":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: state.contrast === "" ? "#FF8C00" : "black",
                            border: state.contrast === "" ? "" : "1px solid white",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        PENDENTE
                    </Paper>
                );
            case "rejected":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: state.contrast === "" ? "#FA8072" : "black",
                            border: state.contrast === "" ? "" : "1px solid white",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        Rejeitado
                    </Paper>
                );
            default:
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: state.contrast === "" ? "#797D7F" : "black",
                            border: state.contrast === "" ? "" : "1px solid white",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        Não requisitado solicitação de professor
                    </Paper>
                )
        }
    };

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData).format("LLL").toString();
    }

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const reloadData = () => {
        setIsLoaded(false)
        getRequest(
            GetAData("users", id),
            (data, header) => {
                setItem(data)
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                setError(true);
            }
        )
    }

    const handleAprove = (userId, userName) => {
        const url = `/users/${userId}/add_teacher`
        const body = {
            "approves": true
        }
        postRequest(
            url,
            body,
            (data) => {
                if (data.errors)
                    HandleSnack("Erro!", true, "warning", "#FA8072");
                else {
                    HandleSnack(`${userName} aceito como professor!`, true, "success", "#228B22");
                    reloadData()
                }
            },
            (error) => {
                HandleSnack("Erro!", true, "warning", "#FA8072");
            }
        )
    }

    const handleReject = (userId, userName) => {
        const url = `/users/${userId}/add_teacher`
        const body = {
            "approves": false
        }
        postRequest(
            url,
            body,
            (data) => {
                if (data.errors)
                    HandleSnack("Erro!", true, "warning", "#FA8072");
                else {
                    HandleSnack(`${userName} rejeitado como professor!`, true, "success", "#228B22");
                    reloadData()
                }
            },
            (error) => {
                HandleSnack("Erro!", true, "warning", "#FA8072");
            }
        )
    }

    const deleteUser = (userId) => {
        deleteRequest(
            `/users/${userId}`,
            (data) => {
                if (data.errors)
                    HandleSnack("Erro!", true, "warning", "#FA8072")
                else {
                    HandleSnack(`${item.name} deletado com sucesso!`, true, "success", "#228B22");
                    history.goBack()
                }
            },
            (error) => {
                HandleSnack("Erro!", true, "warning", "#FA8072")
            }
        )
    }

    const isBlocked = (stateb) => {
        if (stateb === "blocked" || stateb === "banished")
            return <Paper
                style={{
                    textAlign: "center",
                    padding: "0.5em",
                    backgroundColor: state.contrast === "" ? "red" : "black",
                    border: state.contrast === "" ? "" : "1px solid white",
                    fontWeight: "500",
                    color: "#FFFAFA",
                }}
            >
                Bloqueado
            </Paper>
    }

    const actionsForBlockedUser = (stateb) => {
        if (stateb === "blocked" || stateb === "banished") {
            return (
                <StyledButton
                    style={{ width: "100%", marginBottom: "0.5em" }}
                    variant="contained"
                    color="secondary"
                    icon={<RemoveCircleOutlineRoundedIcon />}
                    onClick={() => ReactiveUser()}
                    text="Desbloquear"
                />
            )
        }
        return (
            <Typography>
                Usuário não bloqueado
            </Typography>
        )
    }

    const actionsForStateTeacherRequest = (state) => {
        if (state === "requested") {
            return (
                <Grid container direction="row" spacing={1}>
                    <Grid item>
                        <StyledButton
                            variant="contained"
                            color="secondary"
                            style={{ width: "100%" }}
                            disabled={
                                item.submitter_request === "requested" ? false : true
                            }
                            icon={
                                <CloseRoundedIcon style={{ fill: "#FFFAFA" }} />
                            }
                            onClick={() => { handleReject(item.id, item.name) }}
                            text="Recusar"
                        />
                    </Grid>
                    <Grid item>
                        <StyledButton
                            variant="contained"
                            color="primary"
                            style={{ width: "100%" }}
                            disabled={
                                item.submitter_request === "requested" ? false : true
                            }
                            icon={
                                <CheckRoundedIcon style={{ fill: "#FFFAFA" }} />
                            }
                            onClick={() => { handleAprove(item.id, item.name) }}
                            text="Aceitar"
                        />
                    </Grid>
                </Grid>
            )
        }
        return (
            <Typography>
                Usuário não requisitou conta de professor
            </Typography>
        )
    }

    const ReactiveUser = () => {
        putRequest(
            `/users/${id}/reactivate_user`,
            {},
            (data) => {
                if (data.errors)
                    HandleSnack('Erro ao tentar reativar usuário!', true, 'warning', '#FA8072')
                else {
                    HandleSnack('Usuário foi reativado com sucesso!', true, 'success', '#228B22')
                    reloadData()
                }
            },
            (error) => {
                HandleSnack('Erro ao tentar reativar usuário!', true, 'warning', '#FA8072')
            }
        )
    }

    useEffect(() => {
        getRequest(
            GetAData("users", id),
            (data, header) => {
                setItem(data)
                setIsLoaded(true);
                setError(false);
            },
            (error) => {
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        return (
            <Grid
                container
                spacing={3}
            >
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <Grid
                    item
                    md={6}
                    xs={12}
                >
                    <StyledCard contrast={state.contrast}>
                        <CardContent>
                            <Grid container justify="space-between">
                                <Grid item xs={12}>
                                    <Typography
                                        className={classes.title}
                                        gutterBottom
                                    >
                                        Identificação do usuário
                                    </Typography>
                                </Grid>
                                <Grid container xs={12}>
                                    <Grid item>
                                        <Link
                                            style={{ textDecoration: "none" }}
                                            to={`/admin/usersList`}
                                        >
                                            <StyledButton
                                                icon={<ListRoundedIcon />}
                                                color="primary"
                                                variant="outlined"
                                                text="Listar"
                                            />
                                        </Link>
                                    </Grid>
                                    <Grid item>
                                        <StyledButton
                                            icon={<DeleteRoundedIcon />}
                                            color="secondary"
                                            variant="outlined"
                                            onClick={() => { deleteUser(item.id) }}
                                            text="Deletar"
                                        />
                                    </Grid>
                                    <Grid item>
                                        <Link to={`/admin/EditUser?id=${item.id}`} style={{ textDecoration: "none" }}>
                                            <StyledButton
                                                icon={<EditRoundedIcon />}
                                                color="primary"
                                                variant="outlined"
                                                text="Editar"
                                            />
                                        </Link>
                                    </Grid>
                                </Grid>
                            </Grid>
                            <SizedBox />
                            <Grid container direction="row" justify="flex-start" alignItems="center" spacing={3}>
                                <Grid item justify="center" alignItems="center">
                                    <AvatarDiv>
                                        <img src={item.avatar ? apiDomain + item.avatar : noAvatar} alt='user avatar'
                                            style={{ height: "100%", width: "100%", borderRadius: "50%" }} />
                                    </AvatarDiv>
                                </Grid>
                                <Grid item>
                                    <div className={classes.displayColumn}>
                                        <Typography className={classes.subTitle}>
                                            Nome
                                        </Typography>
                                        <Typography>
                                            {item.name === null ? "Sem dados" : item.name}
                                        </Typography>
                                    </div>
                                    <div className={classes.displayColumn}>
                                        <Typography className={classes.subTitle}>
                                            Email
                                        </Typography>
                                        {
                                            item.email ?
                                            <Link to={`/admin/sendEmail?email=${item.email}`} style={{ textDecoration: 'none' }}>
                                            <StyledButton
                                                variant='text'
                                                color='primary'
                                                icon={<EmailRoundedIcon style={state.contrast === "" ? {color: "#666"} : {color: "white"}}/>}
                                                text={item.email}
                                            />
                                            </Link> : null
                                        }
                                    </div>
                                    <div className={classes.displayColumn}>
                                        <Typography className={classes.subTitle}>
                                            Cpf
                                        </Typography>
                                        <Typography>
                                            {item.cpf === null ? "Sem dados" : item.cpf}
                                        </Typography>
                                    </div>
                                </Grid>
                            </Grid>
                            <Grid>
                                <div className={classes.displayColumn}>
                                    <Typography className={classes.subTitle}>
                                        Escola
                                    </Typography>
                                    <Typography>
                                        {item.school ? item.school.name : "Sem dados"}
                                    </Typography>
                                </div>
                                <div className={classes.displayColumn}>
                                    <Typography className={classes.subTitle}>
                                        Telefone da escola
                                    </Typography>
                                    <Typography>
                                        {item.school ? item.school.phone : "Sem dados"}
                                    </Typography>
                                </div>
                                <div className={classes.displayColumn}>
                                    <Typography className={classes.subTitle}>
                                        UF
                                    </Typography>
                                    <Typography>
                                        {item.school ? item.school.uf : "Sem dados"}
                                    </Typography>
                                </div>
                                <div className={classes.displayColumn}>
                                    <Typography className={classes.subTitle}>
                                        Munícipo
                                    </Typography>
                                    <Typography>
                                        {item.school ? item.school.city : "Sem dados"}
                                    </Typography>
                                </div>
                            </Grid>
                        </CardContent>
                    </StyledCard>
                </Grid>

                <Grid
                    item
                    md={6}
                    xs={12}
                >
                    <Grid item>
                        <StyledCard contrast={state.contrast}>
                            <CardContent>
                                <Typography variant="h5" component="h2">
                                    Informações do perfil
                                </Typography>

                                {stateTeacherRequest(item.submitter_request)}
                                <SizedBox />
                                {isBlocked(item.state)}

                                <SizedBox />

                                <Grid container justify="flex-start">
                                    <Grid item>
                                        <div className={classes.displayColumn}>
                                            <Typography className={classes.subTitle}>
                                                Descrição
                                            </Typography>
                                            <Typography>
                                                {item.description}
                                            </Typography>
                                        </div>
                                    </Grid>

                                    <Grid container direction="row" justify="space-between" spacing={1} alignItems="center">
                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Criado em
                                                </Typography>
                                                <Typography>
                                                    {DisplayDate(item.created_at)}
                                                </Typography>
                                            </div>
                                        </Grid>

                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Atualizado em
                                                </Typography>
                                                <Typography>
                                                    {DisplayDate(item.updated_at)}
                                                </Typography>
                                            </div>
                                        </Grid>
                                    </Grid>

                                    <Grid item>
                                        <div className={classes.displayColumn}>
                                            <Typography className={classes.subTitle}>
                                                Permissões
                                            </Typography>
                                            <Grid container direction="row">
                                                {
                                                    item.roles.map((tag, index) => {
                                                        return (
                                                            <ChipDiv key={index}>
                                                                <Chip label={tag.name} style={state.contrast === "" ? {backgroundColor: "#ddd", color: "black"} : {backgroundColor: "black", color: "white", border: "1px solid white"}} />
                                                            </ChipDiv>
                                                        )
                                                    })
                                                }
                                            </Grid>
                                        </div>
                                    </Grid>

                                    <Grid container direction="row" spacing={1} justify="space-between" alignItems="center">
                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Nota
                                                </Typography>
                                                <Typography>
                                                    {item.score}
                                                </Typography>
                                            </div>
                                        </Grid>

                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Seguidores
                                                </Typography>
                                                <Typography>
                                                    {item.follows_count}
                                                </Typography>
                                            </div>
                                        </Grid>

                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Likes
                                                </Typography>
                                                <Typography>
                                                    {item.likes_count}
                                                </Typography>
                                            </div>
                                        </Grid>

                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Objetos educacionais
                                                </Typography>
                                                <Typography>
                                                    {item.learning_objects_count}
                                                </Typography>
                                            </div>
                                        </Grid>

                                        <Grid item>
                                            <div className={classes.displayColumn}>
                                                <Typography className={classes.subTitle}>
                                                    Coleções
                                                </Typography>
                                                <Typography>
                                                    {item.collections_count}
                                                </Typography>
                                            </div>
                                        </Grid>
                                    </Grid>
                                    <StyledDivider>
                                    </StyledDivider>
                                    <Grid container direction="column">
                                        <Grid item>
                                            <Typography variant="h5" component="h2">
                                                Ações para solicitação de professor
                                            </Typography>
                                        </Grid>
                                        <SizedBox />
                                        {actionsForStateTeacherRequest(item.submitter_request)}
                                    </Grid>
                                    <StyledDivider>
                                    </StyledDivider>
                                    <Grid container direction="column">
                                        <Grid item>
                                            <Typography variant="h5" component="h2">
                                                Ações para usuário bloqueado
                                            </Typography>
                                        </Grid>
                                        <SizedBox />
                                        {actionsForBlockedUser(item.state)}
                                    </Grid>
                                </Grid>
                            </CardContent>
                        </StyledCard>
                    </Grid>
                </Grid>
            </Grid>
        );
    }
};

export default CollectionCard;

const AvatarDiv = styled.div`
    border-radius : 100%;
    height : 126px;
    width : 126px;
    border : 2px solid #d4d4d4;
`
const ChipDiv = styled.div`
    margin-right : 0.5em;
    margin-bottom : 0.5em;
`
const SizedBox = styled.div`
    height : 1em;
`
const StyledDivider = styled.div`
    width : 100%; 
    border-radius : 100%;
    margin-top : 0.6em;
    margin-bottom : 0.6em;
    border : 0.5px solid #d4d4d4;
`