/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from 'react';
//imports material ui components
import CircularProgress from '@material-ui/core/CircularProgress';
import SaveIcon from '@material-ui/icons/Save';
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import Grid from '@material-ui/core/Grid';
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { StyledCard } from "../../../Components/Styles/DataCard";
import { Store } from 'Store'
import FormInput from "Components/FormInput.js"
import StyledButton from '../Button';
//imports services 
import { getRequest, putRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { EditFilter, GetAData } from '../../../Filters';
//Routers
import { Link } from 'react-router-dom';

const NoteVarInputs = () => {

    const { state } = useContext(Store);

    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("id");

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete

    const [isLoading, setIsLoading] = useState(false);
    const [name, setName] = useState()
    const [code, setCode] = useState()
    const [weight, setWeight] = useState()

    const [errorInName, setErrorInName] = useState({
        error: false,
        message: ''
    })

    const [errorInCode, setErrorInCode] = useState({
        error: false,
        message: ''
    })

    const [errorInWeight, setErrorInWeight] = useState({
        error: false,
        message: ''
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
        setName(e.target.value)
    }

    const CodeHandler = (e) => {
        if (errorInCode.error) {
            setErrorInCode({
                error: false,
                message: ''
            })
        }
        setCode(e.target.value)
    }

    const WeightHandler = (e) => {
        if (errorInWeight.error) {
            setErrorInWeight({
                error: false,
                message: ''
            })
        }
        setWeight(e.target.value)
    }

    // Fields
    const fields = [
        {
            label: 'ID',
            value: id,
            default: true,
            type: 'text'
        },
        {
            label: 'Código',
            value: code,
            error: errorInCode.error,
            errorMessage: errorInCode.message,
            onChange: (event) => CodeHandler(event),
            default: true,
            type: 'text'
        },
        {
            label: 'Nome',
            value: name,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event),
            default: false,
            type: 'text',
        },
        {
            label: 'Peso',
            value: weight,
            error: errorInWeight.error,
            errorMessage: errorInWeight.message,
            onChange: (event) => WeightHandler(event),
            default: false,
            type: 'number',
        }
    ]

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const onSubmit = async () => {
        setIsLoading(true)
        const api = EditFilter('scores', id)
        const body = {
            "score": {
                'name': name,
                'code': code,
                'weight': weight
            }
        }
        putRequest(
            api,
            body,
            (data) => {
                if (data.id)
                    HandleSnack('A variável de nota foi atualizada com sucesso!', true, 'success', '#228B22')
                else {
                    if (data.errors) {
                        HandleSnack(`${data.errors[0]}`, true, 'warning', '#FA8072')
                    }
                    else {
                        HandleSnack('Você precisa preencher algumas informações obrigatórias', true, 'warning', '#FFC125')
                        if (data.name) {
                            let errorName = "";
                            data.name.map((err) => (
                                errorName = errorName + err + " e "
                            ))
                            setErrorInName({
                                error: true,
                                message: errorName
                            })
                        }
                        if (data.weight) {
                            let weightError = "";
                            data.weight.map((err) => (
                                weightError = weightError + err + " e "
                            ))
                            setErrorInWeight({
                                error: true,
                                message: weightError
                            })
                        }
                    }
                }
                setIsLoading(false)
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                setIsLoading(false)
            }
        )
    }

    useEffect(() => {
        getRequest(
            GetAData("scores", id),
            (data, header) => {
                setIsLoaded(true);
                setName(data.name)
                setCode(data.code)
                setWeight(data.weight)
            },
            (error) => {
                setIsLoaded(true);
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div>Houve um erro...</div>
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        return (
            <StyledCard contrast={state.contrast}>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>

                    <Grid container xs={12} justify="space-between" alignItems="center" alignContent="center">
                        <Grid>
                            <Typography
                                variant='h4'
                                color="inherit"
                                gutterBottom
                            >
                                {name}
                            </Typography>
                        </Grid>

                        <Grid>
                            <Link style={{ textDecoration: 'none' }} to={`/admin/noteVars`}>
                                <StyledButton
                                    icon={<ListRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                    text="Listar"
                                />
                            </Link>
                        </Grid>
                    </Grid>

                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        {fields.map((field, index) => (
                            <FormInput
                                contrast={state.contrast}
                                key={index}
                                disableField={field.default}
                                error={field.error}
                                help={field.error ? field.errorMessage : ''}
                                style={{ width: '250px', marginBottom: '1em' }}
                                placeholder={field.label}
                                value={field.value}
                                handleChange={field.onChange}
                                inputType={field.type}
                                multi={true}
                            />
                        ))}
                    </form>

                    <div>
                        <StyledButton
                            onClick={() => {
                                onSubmit();
                            }}
                            variant="contained"
                            color="primary"
                            disabled={isLoading}
                            icon={isLoading ? null : <SaveIcon />}
                            text={
                                isLoading ? <CircularProgress size={24} /> : 'Salvar'
                            }
                        />
                    </div>

                </CardContent>
            </StyledCard>
        )
    }
}

export default NoteVarInputs;