/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from "react"
import moment from 'moment'
//imports from local files
import TableData from "../../../Components/Components/Table"
import SnackBar from "../../../../Components/SnackbarComponent"
import { Url } from "../../../Filters"
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import LoadingSpinner from '../../../../Components/LoadingSpinner'
import MobilePageHeader from '../../../Components/Components/MobileComponents/MobilePageHeader'
import PageHeader, { StyledFilter } from '../../../Components/Components/PageHeader'
import { StyledLoadMoreButton, StyledDivButton, useStylesCell, useStylesRow } from "./Collections.js";
import FormInput from "Components/FormInput.js"
//imports from material ui
import TableBody from "@material-ui/core/TableBody"
import TableCell from "@material-ui/core/TableCell"
import MenuItem from "@material-ui/core/MenuItem"
import TableRow from "@material-ui/core/TableRow"
import IconButton from "@material-ui/core/IconButton"
import { Button } from "@material-ui/core"
import CircularProgress from "@material-ui/core/CircularProgress"
import AddRoundedIcon from "@material-ui/icons/AddRounded"
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded"
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded"
import VisibilityIcon from "@material-ui/icons/Visibility"
import AllOutIcon from "@material-ui/icons/AllOut"
//routers
import { Link } from 'react-router-dom'
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"

import { Store } from 'Store'

const Activity = () => {

    const { state } = useContext(Store)

    const classesCell = useStylesCell(state);
    const classesRow = useStylesRow(state);

  const ADD_ONE_LENGHT = [""];
  const TOP_LABELS = [
    "CRIADO EM",
    "DONO(A)",
    "ATIVIDADE",
    "PRIVACIDADE",
    "VISUALIZAR",
  ]; //Labels from Table
  const WINDOW_WIDTH = window.innerWidth

  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
  const [items, setItems] = useState([]); //Necessary to consult the API, data

  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data
  const [showFilter, setShowFilter] = useState(false);
  const [option, setOption] = useState(); //labels of the text field 'to'
  const [currPage, setCurrPage] = useState(0)
  const [forceUpdate, setForceUpdate] = useState(false)

  const [snackInfo, setSnackInfo] = useState({
    message: "",
    icon: "",
    open: false,
    color: "",
  });

  const options = [
    {
      name: "private",
      value: "Privado",
    },
    {
      name: "public",
      value: "Público",
    },
  ];

  //handle snack info
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color,
    });
  };

  const handleChange = (e) => {
    setOption(e.target.value);
  };

  const DisplayDate = (date) => {
    const convertedData = moment.utc(date);
    return moment(convertedData)
      .format("LLL")
      .toString();
  };

  const buildUrl = (privacyOpt) => {
    if (privacyOpt)
      return Url("activities", `"privacy" : "${privacyOpt}"`, currPage, "DESC")

    return Url("activities", "", currPage, "DESC")
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      buildUrl(option),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, option, forceUpdate])

  useEffect(() => {
    setOption()
    setCurrPage(0)
  }, [showFilter])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {

    if (WINDOW_WIDTH <= 820) {
      return <>
        <SnackBar
          severity={snackInfo.icon}
          text={snackInfo.message}
          snackbarOpen={snackInfo.open}
          color={snackInfo.color}
          handleClose={() =>
            setSnackInfo({
              message: "",
              icon: "",
              open: false,
              color: "",
            })
          }
        />

        <MobilePageHeader
          title="Atividades"
          actions={[
            {
              name: "Atualizar",
              isLoading: false,
              func: () => {
                setCurrPage(0)
                setForceUpdate(!forceUpdate)
              },
              icon: <UpdateRoundedIcon />
            },
            {
              name: "Filtrar",
              isLoading: false,
              func: () => {
                setShowFilter(!showFilter);
              },
              icon: <FilterListRoundedIcon />
            }
          ]}
        >
          {showFilter ? (
            <StyledFilter contrast={state.contrast}>

              <div style={{ alignSelf: "flex-end" }}>
                <FormInput
                    contrast={state.contrast}
                    selectable={true}
                    placeholder="Filtro"
                    value={option ? option : ""}
                    handleChange={handleChange}
                    help="Por favor, selecione uma das opções"
                    items={options.map((option, index) => (
                        <MenuItem
                            key={option.value}
                            value={option.name}
                            name={option.value}
                            className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                        >
                            {option.value}
                        </MenuItem>
                    ))}
                />
              </div>
            </StyledFilter>
          ) : null}
        </MobilePageHeader>

        {items.map((row, index) =>
          index === items.length - 1 ? (
            <StyledDivButton
                contrast={state.contrast}
              key={new Date().toISOString() + row.created_at}
            >
              <StyledLoadMoreButton
                contrast={state.contrast}
                key={new Date().toISOString() + row.created_at}
                color="primary"
                variant="text"
                // disabled={isLoadingMoreItems}
                startIcon={<AddRoundedIcon />}
                disabled={isLoadingMoreItems}
                onClick={() => {
                  setCurrPage(currPage + 1)
                }}
              >
                {isLoadingMoreItems ? (
                  <CircularProgress size={24} />
                ) : (
                    "Carregar mais itens"
                  )}
              </StyledLoadMoreButton>
            </StyledDivButton>
          ) : (
              <>
                <MobileList
                  contrast={state.contrast}
                  key={new Date().toISOString() + row.created_at}
                  title={row.id}
                  subtitle={row.privacy}
                  backColor={"#673ab7"}
                  avatar={<AllOutIcon />}
                  href={`/admin/activity?activity=${row.id}`}
                  reset={() => {

                  }}
                  data={
                    [
                      {
                        title: "Dono(a)",
                        subtitle: row.owner ? row.owner.name : "Sem dados"

                      },
                      {
                        title: "Criado em",
                        subtitle: DisplayDate(row.created_at)
                      },
                      {
                        title: "Atividade",
                        subtitle: row.activity
                      }
                    ]
                  }
                />
              </>
            )
        )}
      </>
    }
    else {
      return <>
        <SnackBar
          severity={snackInfo.icon}
          text={snackInfo.message}
          snackbarOpen={snackInfo.open}
          color={snackInfo.color}
          handleClose={() =>
            setSnackInfo({
              message: "",
              icon: "",
              open: false,
              color: "",
            })
          }
        />
        <PageHeader
          title="Atividades"
          actions={[
            {
              name: "Atualizar",
              isLoading: false,
              func: () => {
                setCurrPage(0)
                setForceUpdate(!forceUpdate)
              },
              icon: <UpdateRoundedIcon />
            },
            {
              name: "Filtrar",
              isLoading: false,
              func: () => {
                setShowFilter(!showFilter);
              },
              icon: <FilterListRoundedIcon />
            }
          ]}
        >
          {showFilter ? (
            <StyledFilter contrast={state.contrast}>

              <div style={{ alignSelf: "flex-end" }}>
                <FormInput
                    contrast={state.contrast}
                    selectable={true}
                    placeholder="Filtro"
                    value={option ? option : ""}
                    handleChange={handleChange}
                    help="Por favor, selecione uma das opções"
                    items={options.map((option, index) => (
                        <MenuItem
                            key={option.value}
                            value={option.name}
                            name={option.value}
                            className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                        >
                            {option.value}
                        </MenuItem>
                    ))}
                />
              </div>
            </StyledFilter>
          ) : null}
        </PageHeader>

        <TableData top={TOP_LABELS}>
          <TableBody>
            {items.map((row, index) =>
              index === items.length - 1 ? (
                <TableRow className={classesRow.root} key={new Date().toISOString() + row.created_at} style={{ padding: "1em" }}>
                  {/* Button to load more data */}
                  <TableCell className={classesCell.root} colspan={5}>
                    <Button
                      color="primary"
                      variant="text"
                      // disabled={isLoadingMoreItems}
                      startIcon={<AddRoundedIcon />}
                      disabled={isLoadingMoreItems}
                      onClick={() => {
                        setCurrPage(currPage + 1)
                      }}
                      style={state.contrast === "" ? {width: "100%"} : {width: "100%", color: "yellow", textDecoration: "underline yellow"}}
                    >
                      {isLoadingMoreItems ? (
                        <CircularProgress size={24} />
                      ) : (
                          "Carregar mais itens"
                        )}
                    </Button>
                  </TableCell>
                </TableRow>
              ) : (
                  <TableRow className={classesRow.root} key={new Date().toISOString() + index}>
                    <TableCell className={classesCell.root} component="th" scope="row">
                      {DisplayDate(row.created_at)}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      {
                        row.owner ? row.owner.name : ""
                      }
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">
                      {row.activity}
                    </TableCell>
                    <TableCell className={classesCell.root} align="right">{row.privacy}</TableCell>
                    <TableCell className={classesCell.root} align="right">
                      <Link to={`/admin/activity?activity=${row.id}`}>
                        <IconButton>
                          <VisibilityIcon style={{ fill: state.contrast === "" ? "#00bcd4" : "yellow" }} />
                        </IconButton>
                      </Link>
                    </TableCell>
                  </TableRow>
                )
            )}
          </TableBody>
        </TableData>
      </>
    }
  }
};


export default Activity;

