/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState, useContext } from 'react'
//imports from local files 
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import AlertDialog from "../../../Components/Components/AlertDialog";
import { Url } from '../../../Filters';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { DeleteFilter } from '../../../Filters';
import { getRequest, deleteRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import MobileList from "../../../Components/Components/MobileComponents/SimpleList" 
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader from "../../../Components/Components/PageHeader"
import { StyledLoadMoreButton, StyledDivButton, useStylesCell, useStylesRow } from './Collections';
import { Store } from 'Store'
import StyledButton from '../../../Components/Components/Button';
//imports from material ui 
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import { Button } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded';
import EditRoundedIcon from '@material-ui/icons/EditRounded';
import DeleteRoundedIcon from '@material-ui/icons/DeleteRounded';
//router
import { Link, useHistory } from 'react-router-dom';
import AccountCircleRoundedIcon from "@material-ui/icons/AccountCircleRounded"

const UserPermissions = () => {

    const { state } = useContext(Store)

    const classesCell = useStylesCell(state);
    const classesRow = useStylesRow(state);

  const ADD_ONE_LENGHT = [""];
  const TOP_LABELS = ['ID', 'NOME', 'DESCRIÇÃO', 'AÇÕES'] //Labels from Table 
  const WINDOW_WIDTH = window.innerWidth

  const router = useHistory()

  const [invertList, setInvertList] = useState(false)
  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
  const [items, setItems] = useState([]); //Necessary to consult the API, data 
  const [currPage, setCurrPage] = useState(0)
  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false) //controlls the state of loadind more data
  const [openAlertDialog, setOpenAlertDialog] = useState(false); //controlls the state od alert dialog 

  const [deleteItem, setDeleteItem] = useState({}); //Delete Item
  const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);
  const [forceUpdate, setForceUpdate] = useState(false)

  const [snackInfo, setSnackInfo] = useState({
    message: '',
    icon: '',
    open: false,
    color: '',
  })

  //handle snack info
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color
    })
  }

  //handle Delete
  async function DeleteHandler() {
    const id = deleteItem.id;
    HandleStateAlertDialog(null);
    deleteRequest(
      DeleteFilter("roles", id),
      (data) => {
        if (data.errors)
          HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
        else {
          HandleSnack(
            "A língua foi deletada com sucesso",
            true,
            "success",
            "#228B22"
          );
          setCurrPage(0)
          HandleStateCircularProgress(null);
          removeItemFromList(id)
        }
      },
      (error) => {
        HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
        HandleStateCircularProgress(null);
      }
    )
  }

  const removeItemFromList = (itemId) => {
    let index = -1;
    for (let i = 0; i < items.length; i++) {
      const element = items[i];
      if (element.id === itemId) {
        index = i
        break
      }
    }
    if (index !== -1) {
      const cpyItems = [...items]
      cpyItems.splice(index, 1)
      setItems(cpyItems)
    }
  }

  const HandleStateCircularProgress = (i) => {
    setIsLoadingToDelete(i);
  };

  const HandleStateAlertDialog = (i) => {
    const obj = { ...items[i] };
    setDeleteItem(obj);
    setOpenAlertDialog(!openAlertDialog);
  };

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      Url("roles", "", currPage, invertList ? "ASC" : "DESC"),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, invertList, forceUpdate])

  if (error) {
    return <div>Error: {error.message}</div>;
  } else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {
    if (WINDOW_WIDTH <= 800) {
      return (
        <>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
          <MobilePageHeader
            title="Lista de permissões de usuário"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                  setForceUpdate(!forceUpdate)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/CreateRole')
                },
                icon: <AddRoundedIcon />
              },
              {
                name: "Inverter lista",
                isLoading: false,
                func: () => {
                  cleanArrayAndInvert()
                },
              }
            ]}
          >
          </MobilePageHeader>

          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton contrast={state.contrast} key={new Date().toISOString() + row.created_at}>
                <StyledLoadMoreButton
                  contrast={state.contrast}
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </StyledLoadMoreButton>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    contrast={state.contrast}
                    key={new Date().toISOString() + row.created_at}
                    title={row.name}
                    subtitle={row.id}
                    backColor={"#ff7f00"}
                    avatar={<AccountCircleRoundedIcon />}
                    href={`/admin/EditPermissions?role=${row.id}`}
                    reset={() => {
                    }}
                    data={
                      [
                        {
                          title: "Descrição",
                          subtitle: row.description

                        },
                        {
                          title: "Ações",
                          subtitle: <StyledButton
                            variant="contained"
                            color="secondary"
                            onClick={() => {
                              HandleStateAlertDialog(index)
                              HandleStateCircularProgress(index)
                            }}
                            icon={<DeleteRoundedIcon />}
                            text="Deletar"
                          />
                        },
                      ]
                    }
                  />
                </>
              )
          )}
        </>
      );
    }
    else {
      return (
        <>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() => setSnackInfo({
              message: '',
              icon: '',
              open: false,
              color: ''
            })}
          />

          <PageHeader
            title="Lista de permissões de usuário"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                  setForceUpdate(!forceUpdate)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/CreateRole')
                },
                icon: <AddRoundedIcon />
              }
            ]}
          >
          </PageHeader>

          <TableData
            top={TOP_LABELS}
            onIconPressed={cleanArrayAndInvert}
          >
            <TableBody>
              {items.map((row, index) => (
                index === items.length - 1 ?
                  <TableRow className={classesRow.root} key={new Date().toISOString() + row.created_at}>
                    {/* Button to load more data */}
                    <TableCell className={classesCell.root} colspan={4}>
                      <Button
                        color='primary'
                        variant='text'
                        // disabled={isLoadingMoreItems}
                        startIcon={<AddRoundedIcon />}
                        disabled={isLoadingMoreItems}
                        onClick={() => {
                          setCurrPage(currPage + 1)
                        }}
                        style={state.contrast === "" ? {width: "100%"} : {width: "100%", color: "yellow", textDecoration: "underline yellow"}}
                      >
                        {
                          isLoadingMoreItems ? <CircularProgress size={24} /> : 'Carregar mais itens'
                        }
                      </Button>
                    </TableCell>
                  </TableRow>

                  :

                  <TableRow className={classesRow.root} key={new Date().toISOString() + index}>
                    <TableCell className={classesCell.root} component="th" scope="row">{row.id}</TableCell>
                    <TableCell className={classesCell.root} align="right">{row.name}</TableCell>
                    <TableCell className={classesCell.root} align="right">{row.description}</TableCell>
                    <TableCell className={classesCell.root} align="right">
                      <Link to={`/admin/EditPermissions?role=${row.id}`}>
                        <StyledButton
                          style={{ width: "100%", marginBottom: "0.5em" }}
                          variant="contained"
                          color="primary"
                          icon={<EditRoundedIcon />}
                          text="Editar"
                        />
                      </Link>

                      {isLoadingToDelete === index ? (
                        <CircularProgress size={24} color="primary" />
                      ) : (
                          <StyledButton
                            style={{ width: "100%" }}
                            variant="contained"
                            color="secondary"
                            onClick={() => {
                              HandleStateAlertDialog(index)
                              HandleStateCircularProgress(index)
                            }}
                            icon={<DeleteRoundedIcon />}
                            text="Deletar"
                          />
                        )}

                    </TableCell>
                  </TableRow>
              ))}
            </TableBody>
          </TableData>

          {/* This alert will be displayed if the user click to delete an institution */}
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
        </>
      );
    }
  }
}
export default UserPermissions;