/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/
import React, { useState } from "react";
import GoogleLogin from 'react-google-login'
import { Button } from '@material-ui/core';
//import FacebookLogin from 'react-facebook-login';
import CloseIcon from '@material-ui/icons/Close';
import styled from 'styled-components'
import { device } from './device.js'
import LabeledCheckbox from './Checkbox.js'
import FormInput from "./FormInput.js"
import GoogleLogo from "../img/logo_google.svg"
import ValidateUserInput from './HelperFunctions/FormValidationFunction.js'
import { Link } from 'react-router-dom'
import { apiUrl } from '../env.js'

async function handleGoogleAttempt() {
    let request_url = (
        `${apiUrl}/omniauth/google_oauth2?auth_origin_url=` + window.location.href + '&omniauth_window_type=sameWindow&resource_class=User'
    )
    window.location.replace(request_url)
}

export default function LoginContainer(props) {
    const [formEmail, setEmail] = useState(
        {
            key: false,
            value: localStorage.getItem("@portalmec/uid") || "",
        }
    )

    const [formSenha, setSenha] = useState(
        {
            key: false,
            value: localStorage.getItem("@portalmec/senha") || ""
        }
    )

    const [checkboxControl, setCheckbox] = useState(false)

    const switchModal = (e) => {
        e.preventDefault()
        props.handleClose()
        props.openSignUp()
    }

    const handleChange = (e, type) => {
        const userInput = e.target.value
        const flag = ValidateUserInput(type, userInput)

        if (type === 'email') {
            setEmail({
                ...formEmail,
                key: flag,
                value: userInput
            })
        }
        else if (type === 'password') {
            setSenha({
                ...formSenha,
                key: flag,
                value: userInput
            })
        }
    }

    const limpaCamposForm = () => {
        setEmail({
            ...formEmail,
            key: false,
            value: ''
        });

        setSenha({
            ...formSenha,
            key: false,
            value: ''
        })
    }

    const onSubmit = (e) => {
        e.preventDefault()
        const login = { email: formEmail.value, senha: formSenha.value, checkboxChecked: checkboxControl }

        if (!(formEmail.key || formSenha.key)) {
            props.handleLoginInfo(login)
            limpaCamposForm()
        }

    }

    return (
        <div>
            <ContainerStyled contrast={props.contrast}>
                <DialogHeaderStyled>
                    <span style={{ width: "32px" }} />
                    <H2Styled contrast={props.contrast}>
                        Entrar
                    </H2Styled>
                    <StyledCloseModalButton
                        onClick={props.handleClose}
                        contrast={props.contrast}
                    >
                        <CloseIcon className="icon" />
                    </StyledCloseModalButton>
                </DialogHeaderStyled>

                <DialogContentDiv>
                    <SocialConnectDiv>
                        <GoogleLoginButton contrast={props.contrast} onClick={handleGoogleAttempt}>
                            <img src={GoogleLogo} alt="google-logo" className="google-logo" />
                            <span>Usando o Google</span>
                        </GoogleLoginButton>
                    </SocialConnectDiv>

                    <H3Div>
                        <H3Styled contrast={props.contrast}>
                            <RightSideStrikedH3 contrast={props.contrast} />
                            <span style={{ verticalAlign: "middle" }}>ou</span>
                            <LeftSideStrikedH3 contrast={props.contrast} />
                        </H3Styled>
                    </H3Div>

                    <form onSubmit={e => onSubmit(e)}>
                        <FormInput
                            contrast={props.contrast}
                            inputType={"text"}
                            name={"email"}
                            value={formEmail.value}
                            placeholder={"E-mail"}
                            handleChange={e => handleChange(e, 'email')}
                            required={true}
                            error={formEmail.key}
                            help={formEmail.key ? (formEmail.value.length === 0 ? "Faltou preencher seu e-mail." : "Insira um endereço de e-mail válido") : ""}
                        />
                        <br />
                        <FormInput
                            contrast={props.contrast}
                            inputType={"password"}
                            name={"senha"}
                            value={formSenha.value}
                            placeholder={"Senha"}
                            handleChange={e => handleChange(e, 'password')}
                            required={true}
                            error={formSenha.key}
                            help={formSenha.key ? (formSenha.value.length === 0 ? "Faltou digitar sua senha." : "A senha precisa ter no mínimo 8 caracteres.") : ""}
                        />
                        <br />

                        <RememberRecover contrast={props.contrast}>
                            <LabeledCheckbox contrast={props.contrast} label={<StyledLabel contrast={props.contrast}><StyledSpan>Lembrar-me</StyledSpan></StyledLabel>} handleChange={() => setCheckbox(!checkboxControl)} />
                            <UserForgotTheirPasswordSpan contrast={props.contrast}>Esqueceu a senha? <Link to={"/recuperar-senha"} style={props.contrast === "" ? { textAlign: "right", color: "#4cd0e1" } : { textAlign: "right", color: "yellow", textDecoration: "underline" }} onClick={props.handleClose}>Clique aqui!</Link></UserForgotTheirPasswordSpan>
                        </RememberRecover>

                        <ConfirmContainerStyled>
                            <StyledLoginButton contrast={props.contrast} type="submit" variant="contained">
                                <span style={props.contrast === "" ? { borderRadius: "3px", boxSizing: "border-box", fontFamily: "Roboto, sans serif", fontWeight: "500", color: "#fff" } : { borderRadius: "3px", boxSizing: "border-box", fontFamily: "Roboto, sans serif", fontWeight: "500", color: "yellow" }}>ENTRAR</span>
                            </StyledLoginButton>
                        </ConfirmContainerStyled>
                    </form>

                    <DialogFooterStyled contrast={props.contrast}>
                        <span style={props.contrast === "" ? { textAlign: "center", fontSize: "14px", color: "rgb(102, 102, 102)" } : { textAlign: "center", fontSize: "14px", color: "white" }}>Ainda não tem cadastro? <StyledAnchor contrast={props.contrast} href="" onClick={e => switchModal(e)}>CADASTRE-SE</StyledAnchor></span>
                    </DialogFooterStyled>
                </DialogContentDiv>
            </ContainerStyled>
        </div>
    )
}


const ContainerStyled = styled.div`
    box-sizing : border-box;
    background-color: ${props => props.contrast === "" ? "white" : "black"};
    max-width : none;
    display : flex;
    flex-direction : column;
    min-width : 440px;
    border: 1px solid ${props => props.contrast === "" ? "#666" : "white"};

    align : center;
    padding : 10px;
    border-radius: 4px;
    line-height : 20px;
    font-size : 14px;
    @media ${device.mobileM} {
    width : 100%;
    min-width : unset;
    height : 100%;
    min-width : unset !important;

    }
`

export const DialogHeaderStyled = styled.div`
    text-align : center;
    display : flex;
    flex-direction : row;
    justify-content : space-between;
    padding : 10px 26px 0 26px;
    height : 64px;
`
const H2Styled = styled.h2`
    align-self : center;
    color: ${props => props.contrast === "" ? "#666" : "white"};
    font-size : 26px;
    font-weight : 100;
    font-family: 'Roboto', sans serif, 'Helvetica Neue', Helvetica, Arial, sans-serif !important;
    justify-content: space-between;
    text-align: center;
    letter-spacing: .005em;
`

export const StyledCloseModalButton = styled(Button)`
    display : inline-block;
    position : relative;
    float : right !important;
    margin-right : -8px !important;
    background : transparent !important;
    min-width: 0 !important;
    width : 40px;

    .icon{
        color: ${props => props.contrast === "" ? "#666" : "white"};
    }
`
export const DialogContentDiv = styled.div`
    padding : 20px 30px;
    overflow : visible !important;
`

export const SocialConnectDiv = styled.div`
    margin: auto;

`
export const StyledGoogleLoginButton = styled(GoogleLogin)`
    background-color : #fff !important;
    color : #666 !important;
    border : 1px solid rgb(66, 133, 244);
    box-shadow: 0 0 0 1px #4285f4 !important;
`

const DialogFooterStyled = styled.div`
    box-sizing : border-box;
    font-family : 'Roboto', sans serif;
    margin : 20px -20px;
    padding-top : 20px;
    border-top: ${props => props.contrast === "" ? "1px #e5e5e5 solid" : "1px white solid"};
    justify-content : center;
    text-align : center;
    line-height : 1.42857143
`


export const RightSideStrikedH3 = styled.div`
    display :  inline-block;
    border-bottom: ${props => props.contrast === "" ? "1px dotted #666" : "1px dotted white"};
    vertical-align :  middle;
    font-weight : 500;
    margin-right : 5px;
    width : 44%;
`

export const LeftSideStrikedH3 = styled.div`
    display :  inline-block;
    border-bottom: ${props => props.contrast === "" ? "1px dotted #666" : "1px dotted white"};
    vertical-align :  middle;
    font-weight : 500;
    margin-left : 5px;
    width : 44%;
`

export const H3Div = styled.div`
    margin-top : 0;
    margin-bottom : 10px;
`

const H3Styled = styled.h3`
    overflow : hidden;
    text-align : center;
    font-size : 14px;
    color: ${props => props.contrast === "" ? "#666" : "white"};
    margin : 10px 0;
`

const StyledAnchor = styled.a`
    color: ${props => props.contrast === "" ? "#00bcd4" : "yellow"};
    text-decoration: ${props => props.contrast === "" ? "none" : "underline"};
`
const ConfirmContainerStyled = styled.div`
    display :  flex;
    margin-top : 10px;
    align-items : center;
    justify-content : center;
`
const StyledLoginButton = styled(Button)`
    background-color: ${props => props.contrast === "" ? "#00bcd4 !important" : "black !important"};
    box-shadow : none !important;
    outline: none !important;
    border: ${props => props.contrast === "" ? 0 : "1px solid white !important"};
    text-decoration: ${props => props.contrast === "" ? "none" : "underline !important"};
    color: ${props => props.contrast === "" ? "" : "yellow !important"};
    overflow : hidden !important;
    width : 35% !important;
    display : inline-block !important;
    font-family : 'Roboto', sans serif !important;
    font-size: 14px !important;
    height : 36px !important;
    align-items : center !important;
    border-radius: 3px !important;
    align-self : 50% !important;
`


const RememberRecover = styled.div`
    display : flex;
    justify-content : space-between;
    font-size: 12px;
    font-weight : 400;
`

const StyledLabel = styled.div`
    box-sizing : border-box;
    position : relative;
    vertical-align : middle !important;
    color: ${props => props.contrast === "" ? "#666" : "yellow"};
    text-decoration: ${props => props.contrast === "" ? "none" : "underline"};
`

const UserForgotTheirPasswordSpan = styled.span`
    margin-top : 1em;
    font-size : 12px;
    font-weight : 400;
    text-align : right;
    color: ${props => props.contrast === "" ? "#666" : "white"};
`

const StyledSpan = styled.span`
    font-size : 12px;
    font-weight : 400;
    padding-top : 2px;
`
export const GoogleLoginButton = styled(Button)`
    background-color: ${props => props.contrast === "" ? "#fff !important" : "black !important"};
    box-shadow: ${props => props.contrast === "" ? "0 0 0 1px #4285f4 !important" : "0 0 0 1px white !important"};
    color: ${props => props.contrast === "" ? "#666 !important" : "yellow !important"};
    text-decoration: ${props => props.contrast === "" ? "none !important" : "underline !important"};
    
    font-weight : bolder !important;
    font-size : 12px !important;
    text-transform: none !important;
    width: 40%;

    .google-logo {
        max-height : 18px ;
        padding-right : 5px;

    }

`
