import React, { useState, useEffect } from 'react'
import { Content, ButtonsArea } from './PartOne.js'
import Stepper from '../Stepper.js'
import CustomizedTables from '../Table.js'
import styled from 'styled-components'
import { Button } from '@material-ui/core'
import axios from 'axios'
import { simcaqAPIurl } from '../../env'
import { sortDict } from './PartOne.js'

export default function PartTwo(props) {
    const [schoolList, setSchoolList] = useState([])
    const [state, setState] = useState(''); 
    const [city, setCity] = useState(''); 

    useEffect(() => {
        const code = (props.info.inep_code || '')
        const uf = (props.info.school_uf.abbreviation || '')
        const municipio = (props.info.school_city || '')
        
        setState(uf);
        setCity(municipio);

        if (code !== '') {
            axios.get((`${simcaqAPIurl}/portal_mec_inep?filter=school_cod:` + code)
            ).then((response) => {
                setSchoolList(sortDict(response.data.result))
            }, (error) => console.log('erro ao dar get na escola por inep code')
            )
        }
        else if (uf !== '' && municipio !== '') {
            axios.get((`${simcaqAPIurl}/school?search=state_name:"` + uf + '",city_name:"' + municipio + '"&filter=year:2017')
            ).then((response) => {
                setSchoolList(sortDict(response.data.result))
            }, (error) => console.log('erro ao dar get na escola por uf e municipio', code, uf, municipio)
            )
        }
    }, [])

    const onClickTable = (city_name, id, name, state_name) => {
        props.handleBuscar(city_name, id, name, state_name)
    }

    return (
        <>
            <Content>
                <h4>Vamos localizar o seu cadastro:</h4>
                <Stepper contrast={props.contrast} activeStep={props.activeStep}/>
            </Content>
            <Content>
                <InputContainer>
                    <p>Selecione a sua escola da UF: {state} e do município: {city}:</p>

                    <CustomizedTables
                        contrast={props.contrast}
                        columns={['Codigo INEP', 'Escola', 'UF', 'Cidade']}
                        rows={schoolList} onClickTable={onClickTable}
                    />

                </InputContainer>
            </Content>
            <hr style={props.contrast === "" ? { color: '#666' } : { color: "white" }} />
            <ButtonsArea>
                <ButtonGrey contrast={props.contrast} onClick={() => props.goBack(true, false, false)}>VOLTAR</ButtonGrey>
                <ButtonCancelar contrast={props.contrast} onClick={props.handleCancelar}>CANCELAR VERIFICAÇÃO</ButtonCancelar>
            </ButtonsArea>
        </>
    )
}

export const ButtonCancelar = styled(Button)`
    :hover{
        background: ${props => props.contrast === "" ? "" : "rgba(255,255,0,0.24) !important"};
    }
    background: ${props => props.contrast === "" ? "" : "black !important"};
    border: ${props => props.contrast === "" ? "0 !important" : "1px solid white !important"};
    color: ${props => props.contrast === "" ? "#666 !important" : "yellow !important"};
    text-decoration: ${props => props.contrast === "" ? "none !important" : "underline !important"};
    font-family: 'Roboto',sans-serif !important;
    font-weight: bold !important;
    border-radius: 3px !important;
`

export const ButtonGrey = styled(Button)`
    :hover{
        background: ${props => props.contrast === "" ? "" : "rgba(255,255,0,0.24) !important"};
    }
    background: ${props => props.contrast === "" ? "#fff !important" : "black !important"};
    border: ${props => props.contrast === "" ? "1.5px #666 solid !important" : "1px solid white !important"};
    color: ${props => props.contrast === "" ? "#666 !important" : "yellow !important"};
    text-decoration: ${props => props.contrast === "" ? "none !important" : "underline !important"};
    font-family : 'Roboto',sans-serif !important;
    text-align : center !important;
`

const InputContainer = styled.div`
    max-width : 970px;
    display : block;
    flex : auto;
`
