/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useContext } from 'react'
import { Store } from '../../Store.js'
import styled from 'styled-components'
import Grid from '@material-ui/core/Grid';
import { Button } from '@material-ui/core';
import ErrorIcon from '@material-ui/icons/Error';
import ShareIcon from '@material-ui/icons/Share';
import FolderIcon from '@material-ui/icons/Folder';
import GetAppIcon from '@material-ui/icons/GetApp';
import CallMadeIcon from '@material-ui/icons/CallMade';
import ReportModal from '../ReportModal.js'
import ShareModal from '../ShareModal.js'
import GuardarModal from '../GuardarModal.js'
import RedirectModal from '../RedirectModal'
import MoreVertIcon from '@material-ui/icons/MoreVert';
import Menu from '@material-ui/core/Menu';
import MenuItem from '@material-ui/core/MenuItem';
import LoginModal from './../LoginModal.js'
import SignUpModal from './../SignUpModal'
import { getRequest } from '../HelperFunctions/getAxiosConfig'

import SnackBar from '../SnackbarComponent';

function ReportButton(props) {
    return (
        !props.complained ?
            (
                <ButtonGrey contrast={props.contrast} onClick={props.userLoggedIn ? props.toggleReport : props.openLogin}>
                    <span className="button-text">
                        <ErrorIcon className="icon" />Reportar abuso ou erro
                    </span>
                </ButtonGrey>
            )
            :
            (
                <ButtonGrey contrast={props.contrast}>
                    <span className="button-text-report">
                        <ErrorIcon className="icon" />Você já reportou este recurso
                    </span>
                </ButtonGrey>
            )
    )
}

function DownloadButton(props) {
    return (
        props.downloadableLink ?
            (
                <ButtonOrange contrast={props.contrast} onClick={props.enableDownload}>
                    <span>
                        <GetAppIcon className="icon" />Baixar Recurso
                    </span>
                </ButtonOrange>
            )
            :
            props.link ?
                (
                    <ButtonOrange contrast={props.contrast} onClick={props.toggleRedirect}>
                        <span >
                            <CallMadeIcon className="icon" />Abrir Recurso
                        </span>
                    </ButtonOrange>
                )
                :
                (
                    <ButtonOrange contrast={props.contrast}>
                        <span>
                            Erro
                        </span>
                    </ButtonOrange>
                )
    )
}

export default function Footer(props) {
    const { state } = useContext(Store)
    let windowWidth = window.innerWidth
    const [reportOpen, toggleReport] = useState(false)
    const [shareOpen, toggleShare] = useState(false)
    const [saveToCol, toggleSave] = useState(false)
    const [redirectOpen, toggleRedirect] = useState(false)
    const [loginOpen, setLogin] = useState(false)
    const [successfulLoginOpen, handleSuccessfulLogin] = useState(false)
    const [signUpOpen, setSignUp] = useState(false)
    const [anchorEl, setAnchorEl] = React.useState(null);

    const enableDownload = () => {
        props.handleSnackbar(0)
        const url = props.downloadableLink
        window.open(url, '_blank');

        // there is no error controller here because the router 
        ///:type/:id/download is always returning error
        getRequest(
            `/learning_objects/${props.recursoId}/download`,
            (data, header) => {
            },
            (error) => {
            }
        )
    }

    const handleSignUp = () => {
        setSignUp(!signUpOpen)
    }

    const handleLogin = () => {
        setLogin(!loginOpen)
    }

    const toggleSnackbar = (event, reason) => {
        if (reason === 'clickaway') {
            return;
        }

        handleSuccessfulLogin(false);
    }

    function handleClick(event) {
        setAnchorEl(event.currentTarget);
    }

    function handleClose() {
        setAnchorEl(null);
    }

    return (
        <React.Fragment>
            <SnackBar
                snackbarOpen={successfulLoginOpen}
                handleClose={toggleSnackbar}
                severity={"success"}
                color={"#00acc1"}
                text={"Você está conectado(a)!"}
            />
            {/*-------------------------------MODALS---------------------------------------*/}
            <ReportModal contrast={props.contrast} open={reportOpen} handleClose={() => { toggleReport(false) }}
                form="recurso"
                complainableId={props.recursoId}
                complainableType={"LearningObject"}
            />
            <ShareModal contrast={props.contrast} open={shareOpen} handleClose={() => { toggleShare(false) }}
                thumb={props.thumb} title={props.title} link={props.currPageLink}
            />

            <GuardarModal contrast={props.contrast} open={saveToCol} handleClose={() => { toggleSave(false) }}
                thumb={props.thumb} title={props.title} recursoId={props.recursoId}
            />
            <RedirectModal contrast={props.contrast} open={redirectOpen} handleClose={() => { toggleRedirect(false) }}
                link={props.link}
            />
            <LoginModal contrast={props.contrast} open={loginOpen} handleClose={() => setLogin(false)} openSignUp={handleSignUp}
                openSnackbar={() => { handleSuccessfulLogin(true) }}
            />
            <SignUpModal contrast={props.contrast} open={signUpOpen} handleClose={handleSignUp} openLogin={handleLogin} />
            {/*----------------------------------------------------------------------------*/}

            {/*-----------------------------BUTTONS----------------------------------------*/}
            <OpcoesDiv contrast={props.contrast}>
                <StyledGrid container>
                    {
                        windowWidth > 990 ?
                            (
                                <React.Fragment>
                                    {/*Botao Reportar*/}
                                    <Grid item xs={3}>
                                        <ReportButton
                                            contrast={props.contrast}
                                            userLoggedIn={state.currentUser.id === '' ? false : true}
                                            toggleReport={() => { toggleReport(true) }}
                                            openLogin={handleLogin}
                                            complained={props.complained}
                                        />
                                    </Grid>

                                    {/*Botao Compartilhar*/}
                                    <Grid item xs={3}>
                                        <ButtonGrey contrast={props.contrast} onClick={state.currentUser.id === '' ? () => handleLogin(true) : () => toggleShare(true)}>
                                            <span className="button-text">
                                                <ShareIcon className="icon" />Compartilhar
                                            </span>
                                        </ButtonGrey>
                                    </Grid>

                                    {/*Botao Guardar*/}
                                    <Grid item xs={3}>
                                        <ButtonGrey contrast={props.contrast} onClick={state.currentUser.id === '' ? () => handleLogin(true) : () => toggleSave(true)}>
                                            <span className="button-text">
                                                <FolderIcon className="icon" />Guardar
                                            </span>
                                        </ButtonGrey>
                                    </Grid>

                                    <Grid item xs={3} style={{ justifyContent: "right !important" }}>
                                        <DownloadButton
                                            contrast={props.contrast}
                                            downloadableLink={props.downloadableLink}
                                            link={props.link}
                                            enableDownload={enableDownload}
                                            toggleRedirect={() => { toggleRedirect(true) }}
                                        />
                                    </Grid>
                                </React.Fragment>
                            )
                            :
                            (
                                <React.Fragment>
                                    {/*Botao Guardar*/}
                                    <Grid item xs={4}>
                                        <ButtonGrey contrast={props.contrast} onClick={state.currentUser.id === '' ? () => handleLogin(true) : () => toggleSave(true)}>
                                            <span className="button-text">
                                                <FolderIcon className="icon" />Guardar
                                            </span>
                                        </ButtonGrey>
                                    </Grid>

                                    <Grid item xs={7}>
                                        <DownloadButton
                                            contrast={props.contrast}
                                            downloadableLink={props.downloadableLink}
                                            link={props.link}
                                            enableDownload={enableDownload}
                                            toggleRedirect={() => { toggleRedirect(true) }}
                                        />
                                    </Grid>

                                    <Grid item xs={1}>
                                        <Button aria-haspopup="true" onClick={handleClick} style={props.contrast === "" ? { color: "#666" } : {color: "yellow"}}>
                                            <MoreVertIcon />
                                        </Button>
                                        <Menu
                                            id="simple-menu"
                                            anchorEl={anchorEl}
                                            keepMounted
                                            open={Boolean(anchorEl)}
                                            onClose={handleClose}
                                            MenuListProps={{
                                                disablePadding: true
                                            }}
                                        >
                                            <MenuItem className={`${props.contrast}BackColor`}>
                                                <ReportButton
                                                    contrast={props.contrast}
                                                    userLoggedIn={state.currentUser.id === '' ? false : true}
                                                    toggleReport={() => { toggleReport(true) }}
                                                    openLogin={handleLogin}
                                                    complained={props.complained}
                                                />
                                            </MenuItem>
                                            <MenuItem className={`${props.contrast}BackColor`}>
                                                <ButtonGrey contrast={props.contrast} onClick={state.currentUser.id === '' ? () => handleLogin(true) : () => toggleShare(true)}>
                                                    <span className="button-text">
                                                        <ShareIcon className="icon" /> Compartilhar
                                                    </span>
                                                </ButtonGrey>
                                            </MenuItem>
                                        </Menu>
                                    </Grid>
                                </React.Fragment>
                            )
                    }


                </StyledGrid>
            </OpcoesDiv>
        </React.Fragment>
    )
}

const OpcoesDiv = styled.div`
    margin: 1em 0em;
    display : flex;
    align-items : center;
    height : 65px;
    background-color: ${props => props.contrast === "" ? "white" : "black"};
`

const StyledGrid = styled(Grid)`
    @media screen and (min-width: 990px) {
        padding-left : 12px !important;
    }

    .MuiGrid-item {
        padding-right : 12px;
        padding-left : 12px;
        display : flex;
        justify-content : center;
    }
`

const ButtonGrey = styled(Button)`
    &:hover {
        background-color: ${props => props.contrast === "" ? "rgba(158,158,158,0.2)" : "rgba(255,255,0,0.24)"} !important;
    }
    background-color : ${props => props.contrast === "" ? "transparent" : "black"} !important;
    color: ${props => props.contrast === "" ? "#666" : "yellow"} !important;
    text-decoration: ${props => props.contrast === "" ? "none" : "underline"} !important;
    outline : none !important;
    text-align : center !important;

    .icon {
        color: ${props => props.contrast === "" ? "inherit" : "white"} !important;
        vertical-align : middle !important;
        font-weight : normal !important;
        font-style : normal !important;
        font-size : 24px !important;
        line-height : 1 !important;
        letter-spacing : normal !important;
        text-transform : none !important;
        display : inline-block !important;
        white-space : nowrap !important;
        word-wrap : normal !important;
        direction : ltr !important;
        padding-right : 2px;
    }

    .button-text {
        cursor : pointer;
        line-height : 36px;
        text-align : center;
        white-space : nowrap;
        text-transform : uppercase;
        font-weight : 600;
        font-size : 14px;
        font-style : inherit;
        font-variant : inherit;
    }
    .button-text-report {
        cursor : pointer;
        line-height : 36px;
        text-align : center;
        color : red !important;
        white-space : nowrap;
        text-transform : uppercase;
        font-weight : 600;
        font-size : 14px;
        font-style : inherit;
        font-variant : inherit;
    }
`

const ButtonOrange = styled(Button)`
    &:hover {
        background-color: ${props => props.contrast === "" ? "" : "rgba(255,255,0,0.24) !important"};
    }
    box-shadow : 0 2px 5px 0 rgba(0,0,0,.26) !important;
    background-color: ${props => props.contrast === "" ? "#ff7f00" : "black"} !important;
    color: ${props => props.contrast === "" ? "white" : "yellow"} !important;
    text-decoration: ${props => props.contrast === "" ? "none" : "underline"} !important;
    border: ${props => props.contrast === "" ? "0" : "1px solid white"} !important;
    text-transform : uppercase !important;
    outline : none !important;
    /* max-height: 36px !important; */
    text-align : center !important;
    margin-top : 5px !important;

    .icon {
        color: white !important;
        vertical-align : middle !important;
        font-weight : normal !important;
        font-style : normal !important;
        font-size : 24px !important;
        line-height : 1 !important;
        letter-spacing : normal !important;
        text-transform : none !important;
        display : inline-block !important;
        white-space : nowrap !important;
        word-wrap : normal !important;
        direction : ltr !important;
        padding-right : 2px;
    }

    .text {
        color: ${props => props.contrast === "" ? "white" : "yellow"} !important;
        font-size : 14px;
        font-weight : 600;
    }
`
