/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, {useState, useEffect, useContext} from 'react'
import {Store} from '../Store.js'
import axios from 'axios'
import {apiUrl, apiDomain} from '../env';
import Grid from '@material-ui/core/Grid';
import UploadFileWrapper from '../Components/UploadPageComponents/UploadFileWrapper.js'
import {Redirect} from 'react-router-dom'
import {GreyButton, OrangeButton, InfoBox} from '../Components/UploadPageComponents/StyledComponents.js'
import {Background} from '../Components/UploadPageComponents/StyledComponents'
import LoadingSpinner from '../Components/LoadingSpinner'
import NewTitle from '../Components/UploadPageComponents/Forms/NewTitle.js'
import SobreORecurso from '../Components/UploadPageComponents/Forms/SobreORecurso.js'
import Keywords from '../Components/UploadPageComponents/Forms/Keywords.js'
import Autor from '../Components/UploadPageComponents/Forms/Autor.js'
import TipoDeRecurso from '../Components/UploadPageComponents/Forms/TipoDeRecurso.js'
import Idioma from '../Components/UploadPageComponents/Forms/Idioma.js'
import EducationalStage from '../Components/UploadPageComponents/PartTwoComponents/EducationalStage.js'
import SubjectsAndThemes from '../Components/UploadPageComponents/PartTwoComponents/SubjectsAndThemes.js'
import Licenca from '../Components/UploadPageComponents/PartTwoComponents/Licenca.js'
import {LoadingDiv} from '../Components/UploadPageComponents/PartTwo.js'
import EditThumbnail from '../Components/UploadPageComponents/PartTwoComponents/EditThumbnail.js'
import DisplayThumbnail from '../Components/UploadPageComponents/PartTwoComponents/DisplayThumbnail.js'
import DragAndDropThumbnail from '../Components/UploadPageComponents/PartTwoComponents/DragAndDropThumbnail'
import {getRequest, deleteRequest, putRequest, postRequest} from '../Components/HelperFunctions/getAxiosConfig.js'

import SnackBar from '../Components/SnackbarComponent';

export default function EditLearningObjectPage (props) {
    const recursoId = props.match.params.recursoId
    const {state} = useContext(Store)
    const [learningObject, setLearningObject] = useState({})
    const [publisherDeletedObject, toggleDeleted] = useState(false)
    const [objTypes, setObjTypes] = useState([])
    const [languages, setLanguages] = useState([])
    const [eduStages, setEduStages] = useState([])
    const [subjects, setSubjects] = useState([])

    function handleSuccessfulGet (data) {
        setLearningObject(data)
        if(data.thumbnail) {
            setThumbnail(data.thumbnail)
            setThumbnailStage('done')
        }
        setUpdatedInfo({...updatedInfo, 'id' : data.id})
        toggleLoading(false)
    }

    useEffect( () => {
        const url = `/learning_objects/${recursoId}`
        getRequest(url, handleSuccessfulGet, (error) => {console.log(error)})

        axios.get(`${apiUrl}/object_types/`).then(
            (response) => {
                setObjTypes(response.data.sort((a, b) => (a.name) > (b.name) ? 1 : -1))
            }, (error) => {
                console.log(error)
            }
        )

        axios.get(`${apiUrl}/languages/`).then(
            (response) => {
                setLanguages(response.data)
            }, (error) => {
                console.log(error)
            }
        )

        axios.get(`${apiUrl}/educational_stages/`).then(
            (response) => {
                setEduStages(response.data)
            },
            (error) => {console.log(error)}
        )

        axios.get(`${apiUrl}/subjects/`).then(
            (response) => {
                setSubjects(response.data.sort((a,b) => a.name > b.name ? 1 : -1))
            },
            (error) => {console.log(error)}
        )
    }, [])

    const [loading, toggleLoading] = useState(true)

    const [updatedInfo, setUpdatedInfo] = useState({})

    const onBlurCallback = (fieldName, payload) => {
        const key = fieldName
        let value = payload
        if (key === "tags") {
            value = payload.map( (tag, index) =>
                index = { "name" : tag}
            )
        }
        setUpdatedInfo({...updatedInfo, [fieldName] : value})
    }

    const [snackbarOpen, toggleSnackbar] = useState(false)

    function handleSuccessfulDelete (data) {
        toggleSnackbar(true)
        toggleDeleted(true)
    }
    
    const handleDelete = () => {
        const url = `/learning_objects/${learningObject.id}`

        deleteRequest(url, handleSuccessfulDelete, (error) => {console.log(error)})
    }

    const handleUpdateInfo = () => {
        const url = `/learning_objects/${learningObject.id}`
        let payload = {
            "learning_object" : updatedInfo
        }

        putRequest(url, payload,
            (data) => {props.history.push( `/recurso/${learningObject.id}`)},
            (error) => {console.log(error)}
        )
    }

    const handlePost = () => {
        const url = `/learning_objects/${learningObject.id}/publish`

        let payload = {
            "learning_object" : updatedInfo
        }

        postRequest(url, payload,
            (data) => {props.history.push( `/recurso/${learningObject.id}`)},
            (error) => {console.log(error)}
        )

    }

    const handleSubmit = () => {
        if (state.currentUser.id !== "") {
            const url = `/submissions/`

            let payload = {
                "submission" : {
                    "learning_object_id" : learningObject.id
                }
            }

            postRequest(url, payload,
                (data) => {props.history.push( `/recurso/${learningObject.id}`)},
                (error) => {console.log(error)}
            )

        }
    }

    const checkAccessLevel = (levelToCheck) => {
        if (state.currentUser.id !== '') {
            return (checkUserRole(levelToCheck))
        }
    }

    const checkUserRole = (userRole) => {
        return (state.currentUser.roles.filter((role) => role.name === userRole).length > 0)
    }

    const [thumbnailStage, setThumbnailStage] = useState('default')

    const [thumbnail, setThumbnail] = useState('')
    const [tempUrl, setTempUrl] = useState('')

    const acceptFile = (file) => {
        const objectURL = URL.createObjectURL(file)
        setTempUrl(objectURL)
        setThumbnailStage('editing')
    }
    const updateThumb = (newThumbnail) => {
        setThumbnail(newThumbnail)

    }

    function handleFinalizeThumb (data) {
        if(data.thumbnail) {
            setThumbnail(data.thumbnail)
            setThumbnailStage('done')
        }
    }

    const finalizeThumb = () => {
        setThumbnailStage('uploading')

        const url = `/learning_objects/${learningObject.id}`

        let fdThumb = new FormData()
        fdThumb.set('learning_object[thumbnail]', thumbnail)

        putRequest(url, fdThumb, handleFinalizeThumb, (error) => {setThumbnailStage('default')})
    }

    const handleDeleteThumbnail = () => {
        const url = `/learning_objects/${learningObject.id}`

        let fdThumb = new FormData()
        fdThumb.set('learning_object[thumbnail]', null)

        putRequest(url, fdThumb, (data) => {setThumbnailStage('default')}, (error) => {console.log(error)})
    }

    const chooseRenderStageThumbnail = () => {
        switch(thumbnailStage) {
            case 'uploading':
               return (<LoadingDiv contrast={state.contrast}/>)
            case 'done':
               return (<DisplayThumbnail contrast={state.contrast} acceptFile={acceptFile} thumbnail={`${apiDomain}` + thumbnail} onEditPage={true} handleDelete={handleDeleteThumbnail}/>)
            case 'editing':
               return (<EditThumbnail contrast={state.contrast} finalizeThumb={finalizeThumb} tempImgURL={tempUrl} updateThumb={updateThumb}/>)
            default :
               return (<DragAndDropThumbnail contrast={state.contrast} acceptFile={acceptFile} onEditPage={true}/>)

        }
    }
    return (
        <React.Fragment>
            {
                publisherDeletedObject &&
                <Redirect
                    to={{
                        pathname: "/perfil",
                        state: 1
                    }}
                />
            }
            <SnackBar
                snackbarOpen={snackbarOpen}
                handleClose={() => {toggleSnackbar(false)}}
                severity={"info"}
                color={"#00acc1"}
                text={"Recurso excluido com sucesso!"}
            />
        {
            !loading ?
            (
                <Background contrast={state.contrast}>
                    <div className="container">
                            <Grid container spacing={2}>
                                <Grid item md={4} xs={12}>
                                    <UploadFileWrapper contrast={state.contrast} draftID={recursoId} prevFile={learningObject.attachments ? learningObject.attachments[0] : null}/>
                                </Grid>
                                <Grid item md={8} xs={12}>
                                    <InfoBox contrast={state.contrast}>
                                        <form>
                                            <div className="cabecalho">
                                                <h2>Editar Recurso</h2>
                                            </div>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                {chooseRenderStageThumbnail()}
                                            </Grid>

                                            <Grid container style={{paddingLeft : "15px", paddingRight : "15px"}}>
                                                <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                    <NewTitle contrast={state.contrast} draftID={learningObject.id} initialValue={learningObject.name}
                                                        onBlurCallback={onBlurCallback}
                                                        />
                                                </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <SobreORecurso contrast={state.contrast} draftID={learningObject.id} initialValue={learningObject.description} onBlurCallback={onBlurCallback}/>
                                            </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <Keywords contrast={state.contrast} draftID={learningObject.id} initialValue={learningObject.tags !== undefined ? learningObject.tags.map((tag) => tag.name) : null}
                                                    onBlurCallback={onBlurCallback}/>
                                            </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <Autor contrast={state.contrast} draftID={learningObject.id}
                                                    initialValue={
                                                        learningObject.author === state.currentUser.name ?
                                                        0 : 1
                                                    }
                                                    initialOutroAutor={
                                                        learningObject.author !== state.currentUser.name ?
                                                        learningObject.author : ''
                                                    }
                                                    onBlurCallback={onBlurCallback}
                                                     />
                                            </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <TipoDeRecurso contrast={state.contrast} objTypes={objTypes} draftID={learningObject.id}
                                                    initialValue={learningObject.object_type !== undefined && learningObject.object_type !== null ? objTypes.filter((type) => type.name === learningObject.object_type)[0].id : null} onBlurCallback={onBlurCallback} />
                                            </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <Idioma contrast={state.contrast} languages={languages}  draftID={learningObject.id} 
                                                    initialValue={learningObject.language !== undefined ? learningObject.language.map((language) => language.name) : null} initialIDValues={learningObject.language !== undefined ? learningObject.language.map((language) => language.id) : null}
                                                    onBlurCallback={onBlurCallback} />
                                            </Grid>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <EducationalStage contrast={state.contrast} draftID={learningObject.id} eduStages={eduStages} initialValue={learningObject.educational_stages !== null ? learningObject.educational_stages.map((stage) => String(stage.id)) : null} onBlurCallback={onBlurCallback}
                                                    />
                                            </Grid>

                                            <SubjectsAndThemes contrast={state.contrast} draftId={learningObject.id} subjects={subjects} initialValue={learningObject.subjects !== null ? learningObject.subjects.map(subject => String(subject.id)) : null} onBlurCallback={onBlurCallback}/>

                                            <Grid item xs={12} style={{paddingBottom : "40px"}}>
                                                <Licenca contrast={state.contrast} draftID={learningObject.id} initialValue={learningObject.license ? learningObject.license.id : null} onBlurCallback={onBlurCallback}/>
                                            </Grid>

                                            <Grid item xs={12}>
                                                <div style={{display : "flex", justifyContent : "center"}}>
                                                    <GreyButton contrast={state.contrast} onClick={handleDelete}>
                                                        <span className="button-text">
                                                            EXCLUIR
                                                        </span>
                                                    </GreyButton>

                                                    <GreyButton contrast={state.contrast} onClick={props.history.goBack}>
                                                        <span className="button-text">
                                                            CANCELAR
                                                        </span>
                                                    </GreyButton>

                                                    <OrangeButton contrast={state.contrast} onClick={() => {handleUpdateInfo()}}>
                                                        SALVAR ALTERAÇÕES
                                                    </OrangeButton>

                                                    {/*
                                                        learningObject.state === "draft" &&
                                                            <OrangeButton type="submit">
                                                            PUBLICAR RECURSO
                                                            </OrangeButton>
                                                    */}
                                                    {
                                                        checkAccessLevel("partner") ?
                                                            (
                                                                <OrangeButton contrast={state.contrast} onClick={() => {handleUpdateInfo(); handlePost()}}>PUBLICAR RECURSO</OrangeButton>
                                                            )
                                                            :
                                                            (
                                                                <OrangeButton contrast={state.contrast} onClick={() => {handleUpdateInfo(); handleSubmit()}}>SUBMETER RECURSO</OrangeButton>
                                                            )

                                                    }
                                                </div>
                                            </Grid>

                                            <Grid item xs={12} style={{marginTop : "20px"}}>
                                                <span style={{marginTop : "20px", fontWeight : "200", color : "#a5a5a5", paddingLeft : "10px"}}>
                                                    * Campos obrigatórios
                                                </span>
                                            </Grid>
                                            </Grid>

                                        </form>
                                    </InfoBox>
                                </Grid>
                            </Grid>
                    </div>
                </Background>
            )
            :
            (
                <LoadingSpinner contrast={state.contrast} text={"CARREGANDO"}/>
            )
        }
        </React.Fragment>
    )
}
